package operation

import (
	"context"
	"fmt"
	"testing"
	"time"

	"github.com/stretchr/testify/require"

	"git.perx.ru/perxis/perxis-go/proto/common"

	"github.com/stretchr/testify/mock"

	"git.perx.ru/perxis/perxis-go/proto/mocks"

	"github.com/stretchr/testify/assert"
)

func TestOperation(t *testing.T) {
	op := New(&Proto{})
	require.NotNil(t, op)
	assert.NotEmpty(t, op.Id())
	assert.False(t, op.IsExpired())
	assert.False(t, op.IsDone())
	assert.Less(t, time.Since(op.CreatedAt()), time.Second)

	op.SetRetention(time.Nanosecond)
	time.Sleep(time.Nanosecond)
	assert.True(t, op.IsExpired())

	op.SetCancelFunc(func() {
		// do nothing
	})
}

func TestOperation_Wait(t *testing.T) {
	var count int
	client := mocks.NewOperationServiceClient(t)
	call := client.On("Get", mock.Anything, mock.Anything, mock.Anything)
	call.Run(func(args mock.Arguments) {
		r := args.Get(1).(*common.GetOperationRequest)
		require.Equal(t, "test", r.OperationId)
		p := &Proto{
			Id:          "test",
			Description: fmt.Sprintf("count=%d", count),
		}

		if count == 10 {
			p.Done = true
		}

		call.Return(p, nil)
		count++
	})

	t.Run("Ok", func(t *testing.T) {
		count = 0
		op := Wrap(client, &Proto{
			Id: "test",
		})

		ctx, cancel := context.WithTimeout(context.Background(), time.Second)
		defer cancel()

		err := op.WaitInterval(ctx, time.Millisecond*10)
		assert.NoError(t, err)
	})

	t.Run("Timeout", func(t *testing.T) {
		count = 0
		op := Wrap(client, &Proto{
			Id: "test",
		})

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*30)
		defer cancel()

		err := op.WaitInterval(ctx, time.Millisecond*10)
		assert.ErrorIs(t, err, context.DeadlineExceeded)
	})

	t.Run("Cancel", func(t *testing.T) {
		count = 0
		op := Wrap(client, &Proto{
			Id: "test",
		})

		ctx, cancel := context.WithTimeout(context.Background(), time.Second)
		time.AfterFunc(time.Millisecond*30, cancel)

		err := op.WaitInterval(ctx, time.Millisecond*10)
		assert.ErrorIs(t, err, context.Canceled)
	})
}
